/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: CollectorManager.java msauvage $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.cron;

import javax.servlet.http.HttpServlet;

import org.hsqldb.DatabaseManager;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.util.DataCollectorConfigException;
import org.ow2.petals.tools.webadmin.util.WebConsoleConfigException;
import org.ow2.petals.tools.webadmin.util.WebconsoleConfigHelper;

public final class CollectorManager extends HttpServlet {

    private static final long serialVersionUID = 6616831543123182042L;

    private static CollectorServer collector = null;

    private static String host = null;

    private static String jndiPort = null;

    private static String user = null;

    private static String pwd = null;

    public CollectorManager() {
        super();
    }

    public static void initCollector(String petalsHost, String petalsJndiPort, String petalsUser,
            String petalsPwd) throws DataCollectorConfigException {
        // load data
        host = petalsHost;
        jndiPort = petalsJndiPort;
        user = petalsUser;
        pwd = petalsPwd;
        
        try
        {
            WebconsoleConfigHelper.addProperty("petals-server", "host", petalsHost);
            WebconsoleConfigHelper.addProperty("petals-server", "port", petalsJndiPort);
            WebconsoleConfigHelper.addProperty("petals-server", "user", petalsUser);
            WebconsoleConfigHelper.addProperty("petals-server", "pwd", petalsPwd);
        }
        catch (WebConsoleConfigException e)
        {
            throw new RuntimeException(e);
        }
    }
    

    public static boolean isCollectorInitialized() {
        return (collector != null);
    }

    public static void resetCollector() {
        collector = null;
    }

    public static synchronized CollectorServer getCollector() throws DataCollectorServerException {
        if (collector == null) {        
            try {
                collector = new CollectorServer(host, jndiPort, user, pwd, WebconsoleConfigHelper.getComponentProperties("database"));
            }
            catch (WebConsoleConfigException e)
            {
                throw new RuntimeException(e);
            }
            
        }
        return collector;
    }

    @Override
    public void destroy() {
        if (collector != null) {
            collector.stopQueuesListening();
            collector.stopMessagesListening();
            CollectorServer.stopDatabaseServer();
            DatabaseManager.getTimer().shutDown();
            destroyCollector();
        }
    }

    public static void destroyCollector() {
        collector = null;
    }

}
