/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.administration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.action.AdminAction;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.util.DataCollectorHelper;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * used to change a SA' state
 * 
 * request params : server, sa, action
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class ChangeSAState extends AdminAction {

    private static final String actionTitle = "change service assembly's state";

    final Logger log = Logger.getLogger(ChangeSAState.class);

    @Override
    public ActionForward executeAdminAuthentified(ActionMapping iActionMapping,
            ActionForm iActionForm, HttpServletRequest iServletRequest,
            HttpServletResponse iServletResponse) {

        /** whether request comes from laszlo or not * */
        final String clientType = iServletRequest.getParameter("clientType");

        /** get all request parameters* */
        final String server = iServletRequest.getParameter("server");
        final String subdomain = iServletRequest.getParameter("subdomain");
        final String sa = iServletRequest.getParameter("sa");
        final String action = iServletRequest.getParameter("action");
        final AdminSession adminSession = AdminSession
                .getAdminSession(iServletRequest.getSession());

        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.PETALS_ADMINISTRATION);

        iServletRequest.setAttribute("server", server);
        iServletRequest.setAttribute("subdomain", subdomain);

        /** check mandatory parameters * */
        if (!this.checkMandatoryParameters(iServletRequest,
                new String[] { "server", "sa", "action" }, ChangeSAState.actionTitle)) {
            return iActionMapping.findForward("success");
        }

        try {
            if (StringHelper.isNullOrEmpty(iServletRequest.getParameter("refreshAll"))) {
                final CollectorServer collector = CollectorManager.getCollector();

                if ("stop".equalsIgnoreCase(action)) {
                    collector.getCollectorClient(server).getDeployService().stopServiceAssembly(sa);
                } else if ("start".equalsIgnoreCase(action)) {
                    collector.getCollectorClient(server).getDeployService()
                            .startServiceAssembly(sa);
                } else if ("shutdown".equalsIgnoreCase(action)) {
                    collector.getCollectorClient(server).getDeployService()
                            .shutdownServiceAssembly(sa);
                }

                this.fillInfoBean(adminSession, "State of " + sa + " successfully changed",
                        InfoBean.SUCCESS_MESSAGE);

                /** refresh datas from PEtALS * */
                adminSession.setDomain(DataCollectorHelper.updateDomainInfo(false));
            }
        } catch (final DataCollectorClientException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ChangeSAState.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ChangeSAState.actionTitle, e);
        } catch (final DataCollectorServerException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ChangeSAState.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ChangeSAState.actionTitle, e);
        } catch (DataCollectorClientDoesNotExistException e) {
            this.fillInfoBean(adminSession,
                    "Can't reach PEtALS client, your server list must be out of date",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + actionTitle, e);
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.fillInfoBean(adminSession, "Connection to PEtALS lost, please refresh",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Connection to PEtALS lost : can't " + actionTitle, e);
        }
        if (AdminSession.LASZLO_CLIENT.equalsIgnoreCase(clientType)) {
            return iActionMapping.findForward("laszlo_success");
        } else {
            return iActionMapping.findForward("success");
        }
    }

}
