/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.channelclient;

import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ChannelClientReportKeyNames.CONTENT;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ChannelClientReportKeyNames.MESSAGE_TYPE;

import java.util.HashMap;

import javax.management.Notification;
import javax.management.NotificationListener;

import org.apache.log4j.Logger;
import org.directwebremoting.ScriptSession;
import org.directwebremoting.proxy.dwr.Util;
import org.ow2.petals.jmx.api.IPetalsMonitoringService.ChannelClientReportKeyNames;
import org.ow2.petals.tools.channelclient.jmxclient.ChannelClientJMXClient;
import org.ow2.petals.tools.channelclient.jmxclient.exception.ChannelClientDoesNotExistException;
import org.ow2.petals.tools.channelclient.jmxclient.exception.ChannelClientServiceErrorException;
import org.ow2.petals.tools.channelclient.jmxclient.exception.ConnectionErrorException;
import org.ow2.petals.tools.webadmin.ui.infra.bean.ChannelClientMessageLVO;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * @author mcarpentier
 * 
 */
public class ChannelClientListener implements NotificationListener {
    private String listenedMessageExchangeId;

    private ScriptSession scriptSession;

    final Logger log = Logger.getLogger(SendAjaxMessage.class);

    private ChannelClientJMXClient channelClient;

    /**
     * handle notifications emitted by PEtALS for async messages
     */
    public void handleNotification(Notification arg0, Object arg1) {
        if (listenedMessageExchangeId == null) {
            log.info("Received unexpected message");
        } else {
            log.info("Received message " + arg0.getType());
            if (arg0.getUserData() instanceof HashMap) {
                HashMap<String, Object> msg = (HashMap<String, Object>) arg0.getUserData();
                ChannelClientMessageLVO msgLVO = new ChannelClientMessageLVO();
                String response = StringHelper.prettyPrint((String) msg.get(CONTENT));
                response = StringHelper.HTMLEntityEncode(response);
                msgLVO.setType((String) msg.get(MESSAGE_TYPE));
                msgLVO.setContent(response);
                msgLVO.setDestService((String) msg.get(ChannelClientReportKeyNames.SERVICE));
                msgLVO.setDestInterface((String) msg.get(ChannelClientReportKeyNames.INTERFACE));
                msgLVO.setDestEndpoint((String) msg.get(ChannelClientReportKeyNames.ENDPOINT));
                /** sending to browser, reverse ajax * */
                if (this.scriptSession != null) {
                    log.info("Sending to browser...");
                    Util utilThis = new Util(this.scriptSession);
                    utilThis.addFunctionCall("gotMsgResponse", msgLVO);
                }
                /** remove notification listener from JMX ChannelClient server* */
                if (this.channelClient != null) {
                    try {
                        this.channelClient.getChannelClientServiceClient()
                                .removeNotificationListener(this);
                    } catch (ChannelClientDoesNotExistException e) {
                        log.info("Can't remove listener from JMX server", e);
                    } catch (ConnectionErrorException e) {
                        log.info("Can't remove listener from JMX server", e);
                    } catch (ChannelClientServiceErrorException e) {
                        log.info("Can't remove listener from JMX server", e);
                    }
                }
            } else {
                log.info("Received message has an unknown class");
            }
        }
    }

    public String getListenedMessageExchangeId() {
        return listenedMessageExchangeId;
    }

    public void setListenedMessageExchangeId(String listenedMessageExchangeId) {
        this.listenedMessageExchangeId = listenedMessageExchangeId;
    }

    public void setScriptSession(ScriptSession scriptSession) {
        this.scriptSession = scriptSession;
    }

    public void setChannelClient(ChannelClientJMXClient channelClient) {
        this.channelClient = channelClient;
    }
}
