/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.channelclient;

import java.util.Map;

import org.apache.log4j.Logger;
import org.directwebremoting.WebContext;
import org.directwebremoting.WebContextFactory;
import org.ow2.petals.tools.channelclient.jmxclient.ChannelClientJMXClient;
import org.ow2.petals.jmx.api.IPetalsMonitoringService.ChannelClientReportKeyNames;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.CollectorClient;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.channelclient.exception.WebConsoleChannelClientException;
import org.ow2.petals.tools.webadmin.ui.infra.bean.ChannelClientMessageLVO;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * @author mcarpentier - EBM Websourcing Send a message to PEtALS - called by
 *         DWR (ajax)
 * 
 */
public class SendAjaxMessage {

    private static final String actionTitle = "send Message to PEtALS";

    final Logger log = Logger.getLogger(SendAjaxMessage.class);

    /**
     * Send a message to PEtALS synchronously
     * 
     * @param serverName
     * @param interfaceName
     * @param serviceName
     * @param endpointName
     * @param operationName
     * @param mep
     * @param content
     * @return
     * @throws WebConsoleChannelClientException
     */
    public ChannelClientMessageLVO sendSyncMessage(String serverName, String interfaceName,
            String serviceName, String endpointName, String operationName, String mep,
            String content) throws WebConsoleChannelClientException {

        ChannelClientMessageLVO msg = new ChannelClientMessageLVO();
        try {
            WebContext wctx = WebContextFactory.get();
            AdminSession session = AdminSession.getAdminSession(wctx.getSession());
            log.info("Ajax Request received :\n interface:" + interfaceName + "\n service:"
                    + serviceName + "\n endpoint :" + endpointName);
            /** Role check * */
            if (!session.getUser().isAdmin()) {
                log.info("Ajax user is not admin, skipping action");
                msg = new ChannelClientMessageLVO();
                msg.setContent("You don't have acces to this function.Please log in first");
                msg.setType("forbidden");
            } else {
                final CollectorServer collector = CollectorManager.getCollector();
                CollectorClient cc = collector.getCollectorClient(serverName);
                ChannelClientJMXClient channelClient = cc.getChannelClient();

                Map<String, Object> result = channelClient.getChannelClientServiceClient().sendSync(
                        interfaceName, serviceName, operationName, endpointName, mep, content,
                        null, null, null);

                String response = (String) result.get(ChannelClientReportKeyNames.CONTENT);
                String type = (String) result.get(ChannelClientReportKeyNames.MESSAGE_TYPE);
                String destInterface = (String) result.get(ChannelClientReportKeyNames.INTERFACE);
                String destEndpoint = (String) result.get(ChannelClientReportKeyNames.ENDPOINT);
                String destService = (String) result.get(ChannelClientReportKeyNames.SERVICE);
                log.info("Response received :\n type:" + type + "\n content:" + content);

                response = StringHelper.prettyPrint(response);
                response = StringHelper.HTMLEntityEncode(response);
                msg.setContent(response);
                msg.setType(type);
                msg.setDestEndpoint(destEndpoint);
                msg.setDestInterface(destInterface);
                msg.setDestService(destService);
            }
            /** any exception raised should be treated the same way... * */
        } catch (Exception e) {
            log.error("Can't" + actionTitle + ":", e);
            throw new WebConsoleChannelClientException(e);
        }
        return msg;
    }

    /**
     * Send a message to PEtALS asynchronously
     * 
     * @param serverName
     * @param interfaceName
     * @param serviceName
     * @param endpointName
     * @param operationName
     * @param mep
     * @param content
     * @return
     * @throws WebConsoleChannelClientException
     */
    public void sendASyncMessage(String serverName, String interfaceName, String serviceName,
            String endpointName, String operationName, String mep, String content)
            throws WebConsoleChannelClientException {
        ChannelClientMessageLVO msg = new ChannelClientMessageLVO();
        try {
            WebContext wctx = WebContextFactory.get();
            AdminSession session = AdminSession.getAdminSession(wctx.getSession());
            log.info("Ajax Request received :\n interface:" + interfaceName + "\n service:"
                    + serviceName + "\n endpoint :" + endpointName);

            /** Role check * */
            if (!session.getUser().isAdmin()) {
                log.info("Ajax user is not admin, skipping action");
                msg = new ChannelClientMessageLVO();
                msg.setContent("You don't have acces to this function.Please log in first");
                msg.setType("myType");
            } else {
                final CollectorServer collector = CollectorManager.getCollector();
                CollectorClient cc = collector.getCollectorClient(serverName);
                ChannelClientJMXClient channelClient = cc.getChannelClient();
                ChannelClientListener listener = new ChannelClientListener();
                listener.setScriptSession(wctx.getScriptSession());
                String exchangeId = channelClient.getChannelClientServiceClient().sendASync(listener,
                        interfaceName, serviceName, operationName, endpointName, mep, content,
                        null, null, null);
                listener.setListenedMessageExchangeId(exchangeId);
                listener.setChannelClient(channelClient);
                log.info("Response received :\n exchangeId:" + exchangeId);

            }
            /** any exception raised should be treated the same way... * */
        } catch (Exception e) {
            log.error("Can't" + actionTitle + ":", e);
            throw new WebConsoleChannelClientException(e);
        }
    }

}
