/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.channelclient;

import static org.ow2.petals.tools.webadmin.util.ChannelClientHelper.SE_CLIENT_STATE_NOTFOUND;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.action.ChannelClientAction;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.util.DataCollectorHelper;
import org.ow2.petals.tools.webadmin.util.ChannelClientHelper;
import org.ow2.petals.tools.webadmin.util.ChannelClientHelperException;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * Start the channel client on PEtALS
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class StartChannelClient extends ChannelClientAction {

    private static final String actionTitle = "get PEtALS channel client";

    final Logger log = Logger.getLogger(StartChannelClient.class);

    @Override
    public ActionForward executeChannelClientActionAuthentified(ActionMapping actionMapping,
            ActionForm actionForm, HttpServletRequest servletRequest,
            HttpServletResponse servletResponse) {
        final AdminSession adminSession = AdminSession.getAdminSession(servletRequest.getSession());
        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.CHANNEL_CLIENT);

        final String server = servletRequest.getParameter("server");

        if (StringHelper.isNotNullAndNotEmpty(server)) {
            final DynaActionForm channelClientFormDisplay = (DynaActionForm) actionForm;
            channelClientFormDisplay.set("server", server);
            try {
                if (SE_CLIENT_STATE_NOTFOUND.equalsIgnoreCase(ChannelClientHelper
                        .getChannelClientState(server))) {
                    final ServletContext context = servletRequest.getSession().getServletContext();
                    String reqPath = servletRequest.getContextPath();
                    reqPath = reqPath.substring(reqPath.lastIndexOf("/") + 1);
                    final String realContextPath = context
                            .getRealPath(servletRequest.getContextPath());
                    final String rootPath = realContextPath.substring(0, realContextPath
                            .lastIndexOf(reqPath));
                    ChannelClientHelper.InstallChannelClient(server, servletRequest.getRequestURL()
                            .toString(),rootPath);
                }
                ChannelClientHelper.StartChannelClient(server);
                /** refresh datas from PEtALS * */
                adminSession.setDomain(DataCollectorHelper.updateDomainInfo(true));
            } catch (ChannelClientHelperException e) {
                this.fillInfoBean(adminSession, e.getMessage(), InfoBean.ERROR_MESSAGE);
                this.log.error("Error : can't " + actionTitle, e);
            } catch (DataCollectorServerException e) {
                this.fillInfoBean(adminSession, e.getMessage(), InfoBean.ERROR_MESSAGE);
                this.log.error("Error : can't " + actionTitle, e);
            } catch (DataCollectorClientException e) {
                this.fillInfoBean(adminSession, e.getMessage(), InfoBean.ERROR_MESSAGE);
                this.log.error("Error : can't " + actionTitle, e);
            } catch (DataCollectorServerNotInitializedException e) {
                this.fillInfoBean(adminSession, e.getMessage(), InfoBean.ERROR_MESSAGE);
                this.log.error("Error : can't " + actionTitle, e);
            } catch (DataCollectorClientDoesNotExistException e) {
                this.fillInfoBean(adminSession, e.getMessage(), InfoBean.ERROR_MESSAGE);
                this.log.error("Error : can't " + actionTitle, e);
            }
        }

        return actionMapping.findForward("success");
    }

}
