/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.datacollector;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.hsqldb.DatabaseManager;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.util.DataCollectorConfigException;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.ui.logon.Logon;
import org.ow2.petals.tools.webadmin.util.StringHelper;
import org.ow2.petals.tools.webadmin.util.WebConsoleConfigException;
import org.ow2.petals.tools.webadmin.util.WebconsoleConfigHelper;

/**
 * Display all administrative stuffs for the data collector
 * 
 * @author mcarpentier - EBM Websourcing
 * @author cgirodengo - Capgemini Sud
 * @autor mbeldame - EBM Websourcing
 * 
 */
public class StartStopDatacollector extends Action {

    private static final String propertiesFile = "webconsole.cfg.xml";

    @Override
    public ActionForward execute(ActionMapping iActionMapping, ActionForm form,
            HttpServletRequest iRequest, HttpServletResponse iResponse) {

        final Logger log = Logger.getLogger(Logon.class);

        final DynaActionForm dcForm = (DynaActionForm) form;
        String action = (String) dcForm.get("action");

        if (StringHelper.isNullOrEmpty(action)) {
            action = iRequest.getParameter("action");
        }
        final AdminSession adminSession = AdminSession.getAdminSession(iRequest.getSession());

        /** deals with error messages * */
        final InfoBean infoBean = adminSession.getInfoBean();

        if ("start".equalsIgnoreCase(action)) {
            final String petalsUser = (String) dcForm.get("login");
            final String petalsPort = (String) dcForm.get("port");
            final String petalsHost = (String) dcForm.get("host");
            final String petalsPwd = (String) dcForm.get("password");       

            try {
                if (!CollectorManager.isCollectorInitialized()) {
                    /**
                     * will start the HSQL server if not already started
                     */
                    WebconsoleConfigHelper.init(propertiesFile);
                    CollectorManager.initCollector(petalsHost, petalsPort, petalsUser, petalsPwd);
                    CollectorManager.getCollector().checkDatabaseIsRunning();
                }
            } catch (final DataCollectorServerException e) {
                infoBean.setMessage("Error : "
                        + StringHelper.formatExceptionMessage(e.getMessage()));
                infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
                log.error("Error : login failure", e);
                CollectorManager.destroyCollector();
                return iActionMapping.findForward("failure");
            } catch (final DataCollectorConfigException e) {
                infoBean.setMessage("Error : "
                        + StringHelper.formatExceptionMessage(e.getMessage()));
                infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
                log.error("Error : login failure", e);
                CollectorManager.destroyCollector();
                return iActionMapping.findForward("failure");
            } catch (final WebConsoleConfigException e) {
                infoBean.setMessage("Error : "
                        + StringHelper.formatExceptionMessage(e.getMessage()));
                infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
                log.error("Error : database configuration failure", e);
                CollectorManager.destroyCollector();
                return iActionMapping.findForward("failure");
            }
        } else {
            try {
                if (CollectorManager.isCollectorInitialized()) {
                    final CollectorServer collector = CollectorManager.getCollector();
                    collector.stopQueuesListening();
                    collector.stopMessagesListening();
                    CollectorServer.stopDatabaseServer();
                    DatabaseManager.getTimer().shutDown();
                    CollectorManager.destroyCollector();
                }
            } catch (final DataCollectorServerException e) {
                infoBean.setMessage("Error : "
                        + StringHelper.formatExceptionMessage(e.getMessage()));
                infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
                log.error("Error : login failure", e);
                CollectorManager.destroyCollector();
                return iActionMapping.findForward("failure");
            }
        }
        return iActionMapping.findForward("success");
    }

}
