/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.monitoring;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.action.MonitoringAction;
import org.ow2.petals.tools.webadmin.ui.infra.bean.ComponentLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.DomainLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.bean.QueueLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.ContainerLVO;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.ui.logon.LogonUtil;
import org.ow2.petals.tools.webadmin.util.DataCollectorHelper;

/**
 * @author mcarpentier - EBM Websourcing
 */
public class ShowComponent extends MonitoringAction {

    private static final String actionTitle = "get PEtALS component";

    final Logger log = Logger.getLogger(ShowComponent.class);

    @Override
    public ActionForward executeMonitoringAuthentified(ActionMapping iActionMapping,
            ActionForm iActionForm, HttpServletRequest iServletRequest,
            HttpServletResponse iServletResponse) {

        final AdminSession adminSession = AdminSession
                .getAdminSession(iServletRequest.getSession());

        final String serverName = iServletRequest.getParameter("server");
        final String componentName = iServletRequest.getParameter("component");

        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.MONITORING);
        adminSession.setSelectedComponent(componentName);

        /** check mandatory parameters * */
        if (!this.checkMandatoryParameters(iServletRequest, new String[] { "server", "component",
                "type" }, ShowComponent.actionTitle)) {
            return iActionMapping.findForward("success");
        }

        final DynaActionForm form = (DynaActionForm) iActionForm;

        form.set("serverName", serverName);

        String subdomainName;
        if (iServletRequest.getAttribute("subdomain") != null) {
            subdomainName = (String) iServletRequest.getAttribute("subdomain");
        } else {
            subdomainName = iServletRequest.getParameter("subdomain");
        }

        try {
            final CollectorServer collector = CollectorManager.getCollector();

            if (adminSession.getDomain() == null) {
                adminSession.setDomain(DataCollectorHelper.updateDomainInfo(false));
            }
            DomainLVO nLVO = adminSession.getDomain();

            if (nLVO.getSubdomains().containsKey(subdomainName)) {

                ContainerLVO sLVO = nLVO.getSubdomains().get(subdomainName).getServerLVO(serverName);
                ComponentLVO cLVO = sLVO.getComponent(componentName);

                form.set("name", cLVO.getName());
                form.set("type", cLVO.getType());
                form.set("description", cLVO.getDescription());
                form.set("objectName", cLVO.getObjectName());
                form.set("state", cLVO.getState());

                final List<String> transporters = collector.getCollectorClient(serverName)
                        .getTransportMonitoringService().getTransporters();
                final ArrayList<QueueLVO> queues = new ArrayList<QueueLVO>();

                for (final String transporter : transporters) {
                    final String nameArray[] = transporter.split("-");
                    String formattedQueue = "";
                    if (nameArray.length > 1) {
                        formattedQueue = nameArray[nameArray.length - 1] + " queue";
                    } else {
                        formattedQueue = transporter;
                    }
                    final Map<String, Integer> queueValue = collector
                            .getCollectorClient(serverName).getTransportMonitoringService()
                            .getQueueCurrentSizes(transporter);
                    for (final String key : queueValue.keySet()) {
                        if (key.equalsIgnoreCase(componentName)) {
                            queues.add(new QueueLVO(formattedQueue, "unknown", String
                                    .valueOf(queueValue.get(key))));
                        }
                    }

                }
                form.set("queues", queues);
            } else {
                this.fillInfoBean(adminSession, "Can't " + ShowComponent.actionTitle
                        + " : can't find subdomain " + subdomainName, InfoBean.ERROR_MESSAGE);
            }
        } catch (final DataCollectorClientException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowComponent.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowComponent.actionTitle, e);
        } catch (final DataCollectorServerException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowComponent.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowComponent.actionTitle, e);
        } catch (DataCollectorClientDoesNotExistException e) {
            this.fillInfoBean(adminSession,
                    "Can't reach PEtALS client, your server list must be out of date",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + actionTitle, e);
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.fillInfoBean(adminSession, "Connection to PEtALS lost, please refresh",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Connection to PEtALS lost : can't " + actionTitle, e);
            return LogonUtil.getLogonForward(iServletRequest);
        }

        return iActionMapping.findForward("success");
    }

}
