/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.monitoring;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ow2.petals.tools.webadmin.ui.infra.action.MonitoringAction;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;

/**
 * @author gdecarnin - Open Wide, mcarpentier - EBM WebSourcing
 */
public class ShowExchangesCustom extends MonitoringAction {
    public static String MESSAGES_PER_PAGE = "10";
    
    private static final String actionTitle = "get PEtALS messages exchanges";

    final Logger log = Logger.getLogger(ShowExchanges.class);

    @Override
    public ActionForward executeMonitoringAuthentified(ActionMapping iActionMapping, ActionForm iActionForm,
            HttpServletRequest iServletRequest, HttpServletResponse iServletResponse) {

        AdminSession adminSession = AdminSession.getAdminSession(iServletRequest.getSession());

        /* set navigation info */
     /**   adminSession.setSelectedMenu(AdminSession.MONITORING);

        DynaActionForm form = (DynaActionForm) iActionForm;

        String serverName = iServletRequest.getParameter("server");
        String startMessage = iServletRequest.getParameter("startMessage");
        String messagesPerPage = iServletRequest.getParameter("messagesPerPage");

        if (StringHelper.isNullOrEmpty(startMessage)) {
            startMessage = "0";
        }
        if (StringHelper.isNullOrEmpty(messagesPerPage)) {
            messagesPerPage = MESSAGES_PER_PAGE;
        }

        form.set("serverName", serverName);

        try {
            final CollectorServer collector = CollectorManager.getCollector();

            final List<Map<String, Object>> exchanges = collector.getDefaultCollectorClient()
                    .getMonitoringService().getAllMessageExchanges();

            if (exchanges != null) {

                final List<MessageExchangeLVO> exchangesList = new ArrayList<MessageExchangeLVO>();

                for (final Map<String, Object> re : exchanges) {

                    final String meId = (String) re
                            .get(MessageExchangeLVO.PETALS_MESSAGEEXCHANGE_ID);

                    final MessageExchangeLVO exLVO = new MessageExchangeLVO();
                    exLVO.setExchangeId(meId);
                    exLVO.setMep("unknown");

                    final Set<Map<String, Object>> mesSet = (Set<Map<String, Object>>) re
                            .get(MessageExchangeLVO.MESSAGEEXCHANGE_SNAPSHOTS);

                    if (mesSet != null) {
                        String finalStatus = "unknown";
                        long meStart = Long.MAX_VALUE;
                        long meStop = Long.MIN_VALUE;
                        String service="";
                        for (final Map<String, Object> mes : mesSet) {
                            if (StringHelper.isNullOrEmpty(service)) {
                                service=(String) mes.get(MessageExchangeSnapshotLVO.SERVICE);
                                exLVO.setService(service);
                                exLVO.setOperation((String) mes.get(MessageExchangeSnapshotLVO.OPERATION));
                            }
                            if (mes.get(MessageExchangeSnapshotLVO.COMPONENT) != null) {
                                exLVO.addInvolvedComponent((String) mes.get(MessageExchangeSnapshotLVO.COMPONENT));
                            }
                            if (mes.get(MessageExchangeSnapshotLVO.TIME) != null) {
                                final long time = ((Long) mes.get(MessageExchangeSnapshotLVO.TIME)).longValue();
                                if (time < meStart) {
                                    meStart = time;
                                }
                                if (time >= meStop) {
                                    if (!("error".equalsIgnoreCase(finalStatus))
                                            && !("done".equalsIgnoreCase(finalStatus))) {
                                        finalStatus = (String) mes.get(MessageExchangeSnapshotLVO.STATUS);
                                    }
                                    meStop = time;
                                }
                            }
                        }
                        exLVO.setEndTime(DateHelper.format(new Date(meStop),
                                DateHelper.DETAILLED_DATEFORMAT));
                        exLVO.setStartTime(DateHelper.format(new Date(meStart),
                                DateHelper.DETAILLED_DATEFORMAT));
                        exLVO.setStatus(finalStatus);
                    }
                    exchangesList.add(exLVO);
                }

                // sort by time and service
                Collections.sort(exchangesList, new MessageExchangeLVOComparator());
                Collections.sort(exchangesList, new MessageExchangeLVOServiceComparator());

                // compute stats
                HashMap<String, Integer> serviceAndOperationCounterMap = new HashMap<String, Integer>();
                for (MessageExchangeLVO exchangeLVO : exchangesList) {
                    String serviceAndOperation = exchangeLVO.getService()+"."+exchangeLVO.getOperation();
                    if (serviceAndOperationCounterMap.containsKey(serviceAndOperation)) {
                        int count = serviceAndOperationCounterMap.get(serviceAndOperation);
                        serviceAndOperationCounterMap.put(serviceAndOperation, count + 1);
                    } else {
                        serviceAndOperationCounterMap.put(serviceAndOperation, 1);
                    }
                }

                // sort stats
                ArrayList<Map.Entry<String, Integer>> serviceAndOperationCounterList = new ArrayList<Map.Entry<String, Integer>>();
                serviceAndOperationCounterList.addAll(serviceAndOperationCounterMap.entrySet());
                Collections.sort(serviceAndOperationCounterList, new MapEntryComparator());

                // apply filter on sorted list
                List<MessageExchangeLVO> result = new ArrayList<MessageExchangeLVO>();

                int currentMsg = Integer.parseInt(startMessage);
                int finalMsg = Integer.parseInt(messagesPerPage) + currentMsg - 1;

                for (int i = currentMsg; (i <= finalMsg && (i < exchangesList.size())); i++) {
                    result.add(exchangesList.get(i));
                }

                form.set("startMessage", startMessage);
                form.set("messagesPerPage", messagesPerPage);
                form.set("totalMessages", String.valueOf(exchangesList.size()));

                form.set("exchanges", result);
                form.set("serviceAndOperationCounter", serviceAndOperationCounterList);

            }
        } catch (final DataCollectorClientException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowExchangesCustom.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowExchangesCustom.actionTitle, e);
        } catch (final DataCollectorServerException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowExchangesCustom.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowExchangesCustom.actionTitle, e);
        }**/
        return iActionMapping.findForward("success");
    }

}
