/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.monitoring;

import javax.management.openmbean.CompositeDataSupport;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.action.MonitoringAction;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.ui.logon.LogonUtil;
import org.ow2.petals.tools.webadmin.util.Formatter;

/**
 * @author mcarpentier - EBM Websourcing
 */
public class ShowSystemInfo extends MonitoringAction {

    private static final String actionTitle = "get PEtALS system info";

    final Logger log = Logger.getLogger(ShowSystemInfo.class);

    @Override
    public ActionForward executeMonitoringAuthentified(ActionMapping iActionMapping,
            ActionForm iActionForm, HttpServletRequest iServletRequest,
            HttpServletResponse iServletResponse) {
        final AdminSession adminSession = AdminSession
                .getAdminSession(iServletRequest.getSession());

        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.MONITORING);

        final DynaActionForm form = (DynaActionForm) iActionForm;
        final String server = iServletRequest.getParameter("server");

        try {
            final CollectorServer collector = CollectorManager.getCollector();
            final long uptime = collector.getCollectorClient(server).getSystemInfoService()
                    .getUptime();
            final long daemonThreads = collector.getCollectorClient(server).getSystemInfoService()
                    .getDaemonThreadCount();
            final long peak = collector.getCollectorClient(server).getSystemInfoService()
                    .getPeakThreadCount();
            final long startedThreads = collector.getCollectorClient(server).getSystemInfoService()
                    .getTotalStartedThreadCount();
            final int threadCount = collector.getCollectorClient(server).getSystemInfoService()
                    .getThreadCount();
            final int pendingObjects = collector.getCollectorClient(server).getSystemInfoService()
                    .getPendingObjects();
            final CompositeDataSupport cds = collector.getCollectorClient(server)
                    .getSystemInfoService().getMemoryUsage();
            final long totalPhysicalMemory = collector.getCollectorClient(server)
                    .getSystemInfoService().getTotalPhysicalMemorySize();
            final long freePhysicalMemory = collector.getCollectorClient(server)
                    .getSystemInfoService().getFreePhysicalMemorySize();
            final long processCPUTime = collector.getCollectorClient(server).getSystemInfoService()
                    .getProcessCPUTime();

            if (uptime != -1L) {
                form.set("uptime", Formatter.formatTime(uptime));
            } else {
                form.set("uptime", "unavailable");
            }
            if (daemonThreads != -1L) {
                form.set("daemonThreads", String.valueOf(daemonThreads));
            } else {
                form.set("daemonThreads", "unavailable");
            }
            if (peak != -1L) {
                form.set("peak", String.valueOf(peak));
            } else {
                form.set("peak", "unavailable");
            }
            if (startedThreads != -1L) {
                form.set("startedThreads", String.valueOf(startedThreads));
            } else {
                form.set("startedThreads", "unavailable");
            }
            if (threadCount != -1) {
                form.set("liveThreads", String.valueOf(threadCount));
            } else {
                form.set("liveThreads", "unavailable");
            }
            if (totalPhysicalMemory != -1L) {
                form.set("totalPhysicalMemory", Formatter.formatSize(String
                        .valueOf(totalPhysicalMemory))
                        + " bytes");
            } else {
                form.set("totalPhysicalMemory", "unavailable");
            }
            if (freePhysicalMemory != -1L) {
                form.set("freePhysicalMemory", Formatter.formatSize(String
                        .valueOf(freePhysicalMemory))
                        + " bytes");
            } else {
                form.set("freePhysicalMemory", "unavailable");
            }
            if (processCPUTime != -1L) {
                form.set("processCPUTime", Formatter.formatNanoTime(processCPUTime));
            } else {
                form.set("processCPUTime", "unavailable");
            }

            /** memory info * */
            if (cds.containsKey("used")) {
                form.set("currentHeapSize", Formatter.formatSize(String.valueOf((Long) cds
                        .get("used")))
                        + " bytes");
            } else {
                form.set("currentHeapSize", "unavailable");
            }
            if (cds.containsKey("committed")) {
                form.set("committedMemory", Formatter.formatSize(String.valueOf((Long) cds
                        .get("committed")))
                        + "bytes");
            } else {
                form.set("committedMemory", "unavailable");
            }
            if (cds.containsKey("init")) {
                form.set("initHeapSize", Formatter.formatSize(String
                        .valueOf((Long) cds.get("init")))
                        + " bytes");
            } else {
                form.set("initHeapSize", "unavailable");
            }
            if (cds.containsKey("max")) {
                form.set("maximumHeapSize", Formatter.formatSize(String.valueOf((Long) cds
                        .get("max")))
                        + " bytes");
            } else {
                form.set("maximumHeapSize", "unavailable");
            }
            if (pendingObjects != -1) {
                form.set("pendingObjects", String.valueOf(pendingObjects));
            } else {
                form.set("pendingObjects", "unavailable");
            }

        } catch (final DataCollectorServerException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowSystemInfo.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowSystemInfo.actionTitle, e);
        } catch (final DataCollectorClientException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowSystemInfo.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowSystemInfo.actionTitle, e);
        } catch (DataCollectorClientDoesNotExistException e) {
            this.fillInfoBean(adminSession,
                    "Can't reach PEtALS client, your server list must be out of date",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + actionTitle, e);
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.fillInfoBean(adminSession, "PEtALS connection lost",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Connection to PEtALS lost : can't " + actionTitle, e);
            return LogonUtil.getLogonForward(iServletRequest);
        }
        return iActionMapping.findForward("success");

    }
}
