/**
 * 
 */
package org.ow2.petals.tools.webadmin.ui.proxyservice;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.action.AuthentifiedAction;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.ui.portlet.ShowPortlet;
import org.ow2.petals.tools.webadmin.util.DataCollectorHelper;
import org.ow2.petals.tools.webadmin.util.FilePermissionHelper;

/**
 * @author aruffie
 * 
 */
public class AddProxyService extends AuthentifiedAction {

    final Logger log = Logger.getLogger(ShowPortlet.class);

    private static final String actionTitle = "add Proxy Service";

    @Override
    public ActionForward executeAuthentified(ActionMapping iActionMapping, ActionForm iActionForm,
            HttpServletRequest iServletRequest, HttpServletResponse iServletResponse) {
        /*
         * reset all variable display for errors
         */
        iServletRequest.setAttribute("validate", null);
        iServletRequest.setAttribute("errorProxyName", null);
        iServletRequest.setAttribute("errorWsdl", null);
        iServletRequest.setAttribute("errorAddrSUProvide", null);
        iServletRequest.setAttribute("errorAddrSUConsume", null);
        iServletRequest.setAttribute("errorProcotol", null);
        iServletRequest.setAttribute("errorTimeout", null);
        boolean errors = false;
        DynaActionForm form = (DynaActionForm) iActionForm;
        AdminSession adminSession = AdminSession.getAdminSession(iServletRequest.getSession());

        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.PROXY_SERVICE);

        /** get all form parameters* */
        String proxyName = null;
        String wsdl = null;
        String timeout = null;
        String addrSUProvide = null;
        String addrSUConsume = null;
        Boolean soap = null;
        Boolean rest = null;
        String deployServer = null;
        String endpointName = null;
        String serviceName = null;

        deployServer = form.getString("deployServer");

        /*
         * begin validation of form fields
         */
        if (!(ProxyServiceValidator.validateString(form.getString("proxyName")))) {
            // add Message error if proxyName parameter is null
            iServletRequest.setAttribute("errorProxyName", true);
            errors = true;
        }
        if (!(ProxyServiceValidator.validateString(form.getString("wsdl")))) {
            // add Message error if wsdl parameter null
            iServletRequest.setAttribute("errorWsdl", true);
            errors = true;
        }
        if (!(ProxyServiceValidator.validateString(form.getString("timeout")))) {
            // add Message error if timeout parameter null
            iServletRequest.setAttribute("errorTimeout", true);
            errors = true;
        }
        if (!(ProxyServiceValidator.validateString(form.getString("addrSUProvide")))) {
            // add Message error if addrSUProvide parameter null
            iServletRequest.setAttribute("errorAddrSUProvide", true);
            errors = true;
        }
        if (!(ProxyServiceValidator.validateString(form.getString("addrSUConsume")))) {
            // add Message error if addrSUConsumer parameter null
            iServletRequest.setAttribute("errorAddrSUConsume", true);
            errors = true;
        }
        rest = ProxyServiceValidator.validateBoolean((Boolean) form.get("rest"));
        soap = ProxyServiceValidator.validateBoolean((Boolean) form.get("soap"));

        if ((rest == false) && (soap == false)) {
            iServletRequest.setAttribute("errorProcotol", true);
            errors = true;
        }
        if (!(ProxyServiceValidator.validateString(form.getString("deployServer")))) {

            iServletRequest.setAttribute("errorServer", true);
            errors = true;
        }
        /*
         * end validation of form fields
         */

        // if errors occured return on the page and display errors
        if (errors) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
                    "wsdlFile"));
            return iActionMapping.findForward("error");
        }

        // recover the values of form fields
        proxyName = form.getString("proxyName");
        wsdl = form.getString("wsdl");
        timeout = form.getString("timeout");
        addrSUProvide = form.getString("addrSUProvide");
        addrSUConsume = form.getString("addrSUConsume");
        endpointName = (String) iServletRequest.getSession().getAttribute("endpointName");
        serviceName = (String) iServletRequest.getSession().getAttribute("serviceName");
        
        //create repository and upload file
        final ServletContext context = iServletRequest.getSession().getServletContext();
        String reqPath = iServletRequest.getContextPath();
        reqPath = reqPath.substring(reqPath.lastIndexOf("/")+1);
        final String realContextPath = context.getRealPath(iServletRequest.getContextPath());
        final String rootPath = realContextPath.substring(0, realContextPath.lastIndexOf(reqPath));
        File uploadDir = new File(rootPath + "upload/");
       
        if(!(uploadDir.exists())){
            uploadDir.mkdir();
        }
        try {
            FilePermissionHelper.setRights(uploadDir,new int[] {7,7,7}, false);
            
        } catch (IOException e1) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e1.getMessage());
            e1.printStackTrace();
            return iActionMapping.findForward("error");
            
        }
        
        
        // begin add proxy
        /*
         * Call the handler to add the proxy and generate SA zip file.
         */
        HandlerProxyService handler = null;
        try {
            handler = new HandlerProxyService(addrSUConsume, addrSUProvide,
                    endpointName, proxyName, rest, serviceName, soap, timeout, wsdl, new File(
                            uploadDir.getCanonicalPath()+ "/" + wsdl));
        } catch (IOException e1) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e1.getMessage());
            e1.printStackTrace();
            return iActionMapping.findForward("error");
        }
        handler.setRepository(uploadDir);
        File saFileZip = null;
        try {
            handler.readDescription(handler.readWSDL());
            saFileZip = handler.addProxy();  
        } catch (WSDLException e) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        }

        // end add proxy
        catch (URISyntaxException e) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        } catch (MalformedURLException e) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        } catch (IOException e) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        } catch(RuntimeException e){
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        }

        iServletRequest.setAttribute("wsdlFile", wsdl);

        /*
         * begin deploy SA
         */
        String[] splitedStrings = deployServer.split("\\.");
        String serverName = splitedStrings[2].split("server")[1];
        String subdomain = splitedStrings[1];
        try {
            ArtifactDeployer.saDeployement(iServletRequest, saFileZip, serverName, subdomain);
            // refresh PEtALS domain
            adminSession.setDomain(DataCollectorHelper.updateDomainInfo(false));
        } catch (RuntimeException e) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        } catch (DataCollectorClientException e) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        } catch (DataCollectorServerException e) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        } catch (DataCollectorClientDoesNotExistException e) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        } catch (DataCollectorServerNotInitializedException e) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e.getMessage());
            e.printStackTrace();
            return iActionMapping.findForward("error");
        }

        /*
         * end deploy SA
         */
        iServletRequest.setAttribute("validate", true);
        return iActionMapping.findForward("success");

    }

    
}
