/**
 * 
 */
package org.ow2.petals.tools.webadmin.ui.proxyservice;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FilePermission;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLReader;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription.WSDLVersionConstants;
import org.ow2.petals.tools.webadmin.util.FilePermissionHelper;

import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SaBean;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SuBean;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SuBeanForSa;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.XmlElement;
import com.ebmwebsourcing.commons.jbi.sugenerator.utils.JbiXmlGenerator;
import com.ebmwebsourcing.commons.jbi.sugenerator.utils.JbiZipper;

/**
 * @author aruffie
 * 
 */
public class HandlerProxyService {
    
    // Component used by the proxy SA
    private static final String COMPONENTNAME = new String("petals-bc-soap");

    private File repository = null;
    
    private File wsdlFile = null;

    private String proxyName = null;

    private String wsdl = null;

    private String timeout = null;

    private String addrSUProvide = null;

    private String addrSUConsume = null;

    private Boolean soap = null;

    private Boolean rest = null;

    private String endpointName = null;

    private String serviceName = null;

    // Begin Provider attributs
    private String interfaceNameProvider = null;

    private String interfaceNameNsUrlProvider = null;

    private String endpointNameProvider = null;

    private String namespaceProvider = null;

    private String soapAddr = null;

    private String wsdlVersion = null;

    // End Provider attributs
    public File getWsdlFile() {
        return wsdlFile;
    }

    public String getEndpointNameProvider() {
        return endpointNameProvider;
    }

    public void setEndpointNameProvider(String endpointNameProvider) {
        this.endpointNameProvider = endpointNameProvider;
    }

    public String getNamespaceProvider() {
        return namespaceProvider;
    }

    public void setNamespaceProvider(String namespaceProvider) {
        this.namespaceProvider = namespaceProvider;
    }

    public void setWsdlFile(File wsdlFile) {
        this.wsdlFile = wsdlFile;
    }

    public HandlerProxyService(File wsdlFile) {
        super();
        this.wsdlFile = wsdlFile;
    }

    public String getProxyName() {
        return proxyName;
    }

    public void setProxyName(String proxyName) {
        this.proxyName = proxyName;
    }

    public String getWsdl() {
        return wsdl;
    }

    public void setWsdl(String wsdl) {
        this.wsdl = wsdl;
    }

    public String getTimeout() {
        return timeout;
    }

    public void setTimeout(String timeout) {
        this.timeout = timeout;
    }

    public String getAddrSUProvide() {
        return addrSUProvide;
    }

    public void setAddrSUProvide(String addrSUProvide) {
        this.addrSUProvide = addrSUProvide;
    }

    public String getAddrSUConsume() {
        return addrSUConsume;
    }

    public void setAddrSUConsume(String addrSUConsume) {
        this.addrSUConsume = addrSUConsume;
    }

    public Boolean getSoap() {
        return soap;
    }

    public void setSoap(Boolean soap) {
        this.soap = soap;
    }

    public Boolean getRest() {
        return rest;
    }

    public void setRest(Boolean rest) {
        this.rest = rest;
    }

    public String getEndpointName() {
        return endpointName;
    }

    public void setEndpointName(String endpointName) {
        this.endpointName = endpointName;
    }

    public String getServiceName() {
        return serviceName;
    }

    public void setServiceName(String serviceName) {
        this.serviceName = serviceName;
    }

    public File getRepository() {
        return repository;
    }

    public void setRepository(File repository) {
        this.repository = repository;
    }

    /**
     * Function allow to get Description from a wsdl file
     * 
     * @return Description
     * @throws WSDLException
     * @throws URISyntaxException
     * @throws MalformedURLException
     * @throws PEtALSCDKException
     */
    public Description readWSDL() throws WSDLException, URISyntaxException,
            MalformedURLException {
        WSDLReader wsdlReader = WSDLFactory.newInstance().newWSDLReader();
        return wsdlReader.readWSDL(new URI(wsdlFile.toURI().toString()));
    }

    /**
     * Function allow you to read description
     * 
     * @param description
     */
    public void readDescription(Description description) {
        // Récupération des différentes services
        List servicesList = description.getServices();
        Iterator serviceIt = servicesList.iterator();
        namespaceProvider = description.getTargetNamespace();
        while (serviceIt.hasNext()) {
            Service s = (Service) serviceIt.next();

            List<Endpoint> endpointsList = s.getEndpoints();
            Iterator endpointIt = endpointsList.iterator();
            while (endpointIt.hasNext()) {
                Endpoint e = (Endpoint) endpointIt.next();

                if ((e.getBinding().getInterface().getQName().toString().equals(serviceName))
                        && (e.getName().toString().equals(endpointName))) {
                    endpointNameProvider = endpointName;
                    interfaceNameProvider = e.getBinding().getInterface().getQName().getLocalPart()
                            .toString();
                    interfaceNameNsUrlProvider = e.getBinding().getInterface().getQName()
                            .getNamespaceURI();
                    soapAddr = e.getAddress();
                    if (description.getVersion().equals(WSDLVersionConstants.WSDL11)) {
                        wsdlVersion = "1.1";
                    } else {
                        if (description.getVersion().equals(WSDLVersionConstants.WSDL20)) {
                            wsdlVersion = "2.0";
                        }
                    }

                }
            }

        }
    }

    public HandlerProxyService(String addrSUConsume, String addrSUProvide, String proxyName,
            Boolean rest, Boolean soap, String timeout, String wsdl, File wsdlFile) {
        super();
        this.addrSUConsume = addrSUConsume;
        this.addrSUProvide = addrSUProvide;
        this.proxyName = proxyName;
        this.rest = rest;
        this.soap = soap;
        this.timeout = timeout;
        this.wsdl = wsdl;
        this.wsdlFile = wsdlFile;

    }

    /**
     * Function allow you to create the proxy with informations content in the
     * handler, and return the SA zip file
     * 
     * @return
     * @throws IOException 
     */
    public File addProxy() throws IOException {

        ArrayList<XmlElement> componentElementsConsumer = new ArrayList<XmlElement>();
        ArrayList<XmlElement> componentElementsProvider = new ArrayList<XmlElement>();
        List<SuBean> suBeans = new ArrayList<SuBean>();

        /*
         * Cut the LocalName and Namespace form the QName to avoid duplication
         * in jbi consume service-name
         */
        String LocalName = serviceName
                .substring(serviceName.indexOf("}") + 1, serviceName.length());

        /*
         * Begin Provide SOAP parameter
         */
        XmlElement element = new XmlElement();
        element.setName("soap:address");
        element.setValue(soapAddr);
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:address");
        element.setValue(addrSUConsume);
        componentElementsConsumer.add(element);

        element = new XmlElement();
        element.setName("soap:soap-version");
        element.setValue(wsdlVersion);
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:add-root");
        element.setValue("false");
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:chunked-mode");
        element.setValue("false");
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:cleanup-transport");
        element.setValue("true");
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:mode");
        element.setValue("SOAP");
        componentElementsProvider.add(element);
        /*
         * Ending Provide SOAP parameter
         */

        File wsdlProvide = null;
        try {
            wsdlProvide = modifySoapAddressWSDL();
        } catch (final IOException e1) {
            System.out.println("Error occured durring the creation of the new wsdl"
                    + " for the provide: " + e1.getMessage());
        }

        // Create the Consumer/Provider SU
        SuBean suConsume = SuCreator.createSuBeanConsume(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", addrSUConsume, addrSUConsume, addrSUConsume, "ns", "operation", timeout,
                componentElementsConsumer);
        SuBean suProvide = SuCreator.createSuBeanProvide(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", interfaceNameProvider, interfaceNameProvider, interfaceNameProvider,
                namespaceProvider, "operation", timeout, componentElementsProvider, wsdlProvide
                        .getName());

        // Add these SU in the su list of the SA
        suBeans.add(suConsume);
        suBeans.add(suProvide);

        // Generate a jbi.xml file for a service assembly.
        SaBean saBean = new SaBean();
        saBean.setSaName(proxyName);
        saBean.setDescription("SA generated for the " + proxyName);
        List<SuBeanForSa> sus = new ArrayList<SuBeanForSa>();
        List<File> suZipFiles = new ArrayList<File>();
        for (int i = 0; i < suBeans.size(); i++) {

            SuBean suBean = suBeans.get(i);
            // Generate a jbi.xml file a service unit.
            String jbiXmlForSu = JbiXmlGenerator.getInstance().generateJbiXmlFileForSu(suBean);

            File tempSaDirectory = new File(repository.getCanonicalPath());
            if (!tempSaDirectory.exists()) {
                tempSaDirectory.mkdir();
            }
            try {
                FilePermissionHelper.setRights(tempSaDirectory,new int[] {7,7,7}, false);
                } catch (final IOException e1) {
                e1.printStackTrace();
                throw new RuntimeException(e1.getMessage());
            }
            
            File jbiSu = new File(repository.getCanonicalPath() + "/jbi.xml");
            File suFile = null;
            try { 
                FileWriter writer = new FileWriter(jbiSu);
                writer.write(jbiXmlForSu);
                writer.close();
                List<File> rootFiles = new ArrayList<File>();

                JbiZipper jbiZipper = JbiZipper.getInstance();
                // Save SU in the repository
                try {
                    
                    if (i == 0) {
                        rootFiles.add(jbiSu);
                        suFile = new File(repository.getCanonicalPath() + "/" + addrSUConsume + ".zip");
                    } else {
                        rootFiles.add(jbiSu);
                        rootFiles.add(wsdlProvide);
                        suFile = new File(repository.getCanonicalPath() + "/" + addrSUProvide + ".zip");
                    }
                    
                    suZipFiles.add(jbiZipper.createSuZipFile(suFile, rootFiles));
                    
                }catch (FileNotFoundException e) {
                    System.out.println("FilenotFoundException in suFile initialization: " + e.getMessage());
                    e.printStackTrace();
                    throw new RuntimeException(e.getMessage());
                } catch (Exception e) {
                    System.out.println("Exception in suFile initialization: " + e.getMessage());
                    e.printStackTrace();
                    throw new RuntimeException(e.getMessage());
                }

            } catch (IOException e) {
                System.out.println("IOException occured when "+jbiSu.getName()+" was wrote");
                e.printStackTrace();
            }

            SuBeanForSa suBeanForSa = new SuBeanForSa(suBean);
            suBeanForSa.setZipArtifact(suFile.getName());
            
            if (i == 1) {
                suBeanForSa.setSuName(addrSUProvide);
              
            } else {
                suBeanForSa.setSuName(addrSUConsume);
                
            }
            suBeanForSa.setComponentName(COMPONENTNAME);
            
            sus.add(suBeanForSa);

            jbiSu.delete();

        }

        /*
         * witch the order of the su, because the provider su need to be the
         * first in the sa jbi file
         */
        sus = switchSu(sus);
        saBean.setSus(sus);

        // Generate jbi file for the SA
        String jbiXmlForSa = JbiXmlGenerator.getInstance().generateJbiXmlFileForSa(saBean);
        File jbiSa = new File(repository.getCanonicalPath() + "/jbi.xml");
        File saZipFile = null;
        // Création des zip de la SA
        try {         
                FileWriter writer = new FileWriter(jbiSa);
                writer.write(jbiXmlForSa);
                writer.close();
                JbiZipper jbiZipper = JbiZipper.getInstance();

                // Save the SA zip file in the repository
                File saFile = new File(repository.getCanonicalPath() + "/" + proxyName + ".zip");
                saZipFile = jbiZipper.createSaZipFile(saFile, suZipFiles, jbiSa);            
        } catch (IOException e) {
            System.out.println("IOException occured when "+jbiSa.getName()+" was wrote");
            e.printStackTrace();
            throw new RuntimeException(e.getMessage());
        }   
        jbiSa.delete();
        return saZipFile;
    }

    public HandlerProxyService(String addrSUConsume, String addrSUProvide, String endpointName,
            String proxyName, Boolean rest, String serviceName, Boolean soap, String timeout,
            String wsdl, File wsdlFile) {
        super();
        this.addrSUConsume = addrSUConsume;
        this.addrSUProvide = addrSUProvide;
        this.endpointName = endpointName;
        this.proxyName = proxyName;
        this.rest = rest;
        this.serviceName = serviceName;
        this.soap = soap;
        this.timeout = timeout;
        this.wsdl = wsdl;
        this.wsdlFile = wsdlFile;
    }

    /**
     * Allow to switch the SU in the list
     * 
     * @param sus
     * @return
     */
    private List<SuBeanForSa> switchSu(List<SuBeanForSa> sus) {
        List<SuBeanForSa> susList = new ArrayList<SuBeanForSa>();
        susList.add(sus.get(1));
        susList.add(sus.get(0));
        return susList;
    }

    /**
     * Return the new wsdlFile with the new address soap for the provide SU
     * 
     * @return
     * @throws IOException
     */
    private File modifySoapAddressWSDL() throws IOException {
        
        String fileName = wsdlFile.getName().substring(0, wsdlFile.getName().indexOf(".")); 
        File wsdlProvide = new File(repository.getCanonicalPath()+ "/" + fileName + "Provide.wsdl");
        if (!wsdlProvide.exists()) {
            wsdlProvide.createNewFile();
        }
        FilePermissionHelper.setRights(wsdlProvide,new int[] {7,7,7},false);
        
        BufferedReader reader = null;
        BufferedWriter writer = null;
        String line = "";

        reader = new BufferedReader(new FileReader(wsdlFile));
        writer = new BufferedWriter(new FileWriter(wsdlProvide));
        while ((line = reader.readLine()) != null) {
            Pattern pattern = Pattern.compile("soap:address");
            Matcher matcher = pattern.matcher(line);
            boolean find = matcher.find();
            if (find) {
                writer.write("<soap:address location=\"" + addrSUProvide + "\"/>\n");
                writer.flush();
            } else {
                writer.write(line + "\n");
                writer.flush();
            }
        }
        writer.close();
        reader.close();
        return wsdlProvide;
    }
}
