package org.ow2.petals.tools.webadmin.ui.proxyservice;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.bean.DomainLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.util.DataCollectorHelper;
import org.ow2.petals.tools.webadmin.util.FilePermissionHelper;

public class ShowMenu extends ProxyServiceAction {
    private static final String actionTitle = "get PEtALS proxys";

    final Logger log = Logger.getLogger(ShowMenu.class);

    @Override
    public ActionForward executeProxyServiceActionAuthentified(ActionMapping iActionMapping,
            ActionForm iActionForm, HttpServletRequest iServletRequest,
            HttpServletResponse iServletResponse) {

        final AdminSession adminSession = AdminSession
                .getAdminSession(iServletRequest.getSession());

        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.PROXY_SERVICE);

        final DynaActionForm form = (DynaActionForm) iActionForm;

        try {

            if (adminSession.getDomain() == null) {
                adminSession.setDomain(DataCollectorHelper.updateDomainInfo(true));
            }

            DomainLVO nLVO = adminSession.getDomain();

            form.set("domain", nLVO);

        } catch (final DataCollectorClientException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowMenu.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowMenu.actionTitle, e);
        } catch (final DataCollectorServerException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowMenu.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowMenu.actionTitle, e);
        } catch (DataCollectorClientDoesNotExistException e) {
            this.fillInfoBean(adminSession,
                    "Can't reach PEtALS client, your server list must be out of date",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + actionTitle, e);
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.fillInfoBean(adminSession, "Connection to PEtALS lost, please refresh",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Connection to PEtALS lost : can't " + actionTitle, e);
        }

        String wsdl = (String) iServletRequest.getAttribute("wsdlFile");
        if (wsdl == null) {
            wsdl = (String) iServletRequest.getSession().getAttribute("wsdlFile");
        }
        File wsdlFile = null;
        try{
            wsdlFile = new File(checkRepository(iServletRequest, iServletResponse) + "/" + wsdl);
        } catch (IOException e1) {
            iServletRequest.setAttribute("error", e1.getMessage());
            e1.printStackTrace();
            return iActionMapping.findForward("error");
        } catch (RuntimeException e1) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
            "wsdlFile"));
            iServletRequest.setAttribute("error", e1.getMessage());
            e1.printStackTrace();
            return iActionMapping.findForward("error");
        }
        try {
            iServletRequest.setAttribute("wsdlServices", HandlerAddWsdl.getServices(wsdlFile));
        } catch (FileNotFoundException e) {
            iServletRequest.setAttribute("errorWsdlFile", true);
            System.out.println("Exception occured for get the wsdl file: " + e.getMessage());
            return iActionMapping.findForward("error");
        }
        // end parse wsdl file with easy wsdl

        iServletRequest.setAttribute("wsdlFile", wsdl);

        return iActionMapping.findForward("success");

    }
    
    
    private String checkRepository( HttpServletRequest iServletRequest,
            HttpServletResponse iServletResponse) throws IOException{
        //create repository and upload file
        final ServletContext context = iServletRequest.getSession().getServletContext();
        String reqPath = iServletRequest.getContextPath();
        reqPath = reqPath.substring(reqPath.lastIndexOf("/")+1);
        final String realContextPath = context.getRealPath(iServletRequest.getContextPath());
        final String rootPath = realContextPath.substring(0, realContextPath.lastIndexOf(reqPath));
        File uploadDir = new File(rootPath + "upload/");
       
        if(!(uploadDir.exists())){
            uploadDir.mkdir();
        }
        try {
            FilePermissionHelper.setRights(uploadDir,new int[] {7,7,7}, false);
        } catch (IOException e1) {
            e1.printStackTrace();
            throw new RuntimeException(e1.getMessage());        
        }
        return uploadDir.getCanonicalPath();
    }
}
