/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.util;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.petals.tools.webadmin.datacollector.util.DataCollectorConfigException;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * @author mcarpentier
 * 
 * Load an XML configuration file
 */
public class ConfigHelper {

    /**
     * Read the portal configuration Document
     * 
     * @return the portal configuration Document
     * @throws DataCollectorConfigException
     *             if configuration file was not found
     */
    public static Document getConfig(String XMLfile) throws WebConsoleConfigException {
        Document doc = null;
        String myFile = XMLfile;
        if (StringHelper.isNullOrEmpty(XMLfile)) {
            myFile = "datacollector.cfg.xml";
        }
        final InputStream myInputStream = ConfigHelper.class.getResourceAsStream("/" + myFile);
        if (myInputStream == null) {
            // TODO : log
            throw new ExceptionInInitializerError("Unable to find configuration file");
        }
        doc = ConfigHelper.parseXmlFile(myInputStream);
        return doc;

    }

    /**
     * Parse an xml file to produce a Document object mapping this xml template
     * 
     * @param xmlURL
     *            the InputStream loaded from the XML document
     * @return the JDOM document
     * @throws DataCollectorConfigException
     */
    private static Document parseXmlFile(InputStream xmlURL) throws WebConsoleConfigException {
        Document document = null;
        // Create the DocumentBuilderFactory
        final DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
        // Create the DocumentBuilder
        DocumentBuilder docbuilder;
        try {
            docbuilder = dbfactory.newDocumentBuilder();
            document = docbuilder.parse(xmlURL);
        } catch (final ParserConfigurationException e) {
            throw new WebConsoleConfigException(e);
        } catch (final SAXException e) {
            throw new WebConsoleConfigException(e);
        } catch (final IOException e) {
            throw new WebConsoleConfigException(e);
        }
        return document;
    }

    /**
     * TODO : check if XML file is following its DTD ?
     */
    public boolean isValid(String XMLfile) {
        return true;
    }

}
