/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM WebSourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.util;


import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author mcarpentier
 * @autor mbeldame - EBM Websourcing
 * 
 */

public class WebconsoleConfiguration {
    
    //A map to store a properties object by component
    private final Map<String,Properties> propertiesList =new HashMap<String,Properties>();
    
    private static final Logger log = Logger.getLogger(WebconsoleConfiguration.class);

    public WebconsoleConfiguration(Document doc) {
        readXmlProperties(doc);
    }

    /**
     * @param the component : datacollector-configuration,...
     * @param the property to retrieve
     * @return a String object containing the value of the property. null if the component or the property does not exist.
     */
    public String getProperty(String component, String property) {
        String result = null;
        if (this.propertiesList.containsKey(component)){
            result = ((Properties) this.propertiesList.get(component)).getProperty(property);
        }
        return result;
    }
    
    /**
     * @param the component (datacollector-configuration,...) containing the property.
     * @param the propertyName
     * @param the property value
     * 
     */
    public void addProperty(String component, String propertyName, String propertyValue)
    {    
        if ( this.propertiesList.get(component) == null )
        {
            Properties props = new Properties();
            props.setProperty(propertyName, propertyValue);
            this.propertiesList.put(component, props);
        }else
        {
            ((Properties)this.propertiesList.get(component)).setProperty(propertyName, propertyValue);
        }
    }
    
    /**
     * @param the XMl Document containing the properties to load.
     */
    public void readXmlProperties(Document doc)
    {
        Node rootElement = doc.getFirstChild();
        final NodeList list = rootElement.getChildNodes();
        
        for (int i=0; i<list.getLength(); i++)
        {
            Node currentNode = list.item(i);   
            if (currentNode instanceof Element)
            {
                String componentName = currentNode.getNodeName();
                Properties props;
                
                if ( this.propertiesList.get(componentName) == null )
                {
                    log.log(Level.DEBUG,"Trying to add component "+componentName);
                    props = new Properties();
                    this.propertiesList.put(componentName, props);
                }else
                {
                    props = (Properties)this.propertiesList.get(componentName);
                }
                
                NodeList subnodes = currentNode.getChildNodes();
                
                for (int j=0; j<subnodes.getLength(); j++)
                { 
                    Node currentSubNode = subnodes.item(j);
                    
                    if (currentSubNode instanceof Element){
                        String propertyName=currentSubNode.getNodeName();
                        Node lastNode = currentSubNode.getLastChild();
                        String propertyValue ="";
                        
                        if (lastNode != null){
                            propertyValue =lastNode.getNodeValue();
                        }
                        
                        log.log(Level.DEBUG,"Trying to add property "+propertyName+" with value "+propertyValue+" in component " + componentName);
                        props.put(propertyName,propertyValue);
                    }
                }
            }
        }
    }
    
    /**
     * @param the  component : datacollector-configuration,...
     * @return a Properties object containing all properties name and value of the component. null if the component does not exist.
     */
    public Properties getComponentProperties(String component)
    {
        Properties props = null;
        if (this.propertiesList.containsKey(component))
        {
            props = this.propertiesList.get(component);
        }
        return props;
    }
    
}
